/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 6.2 (Monotone-bounded, p. 128)
 *
 * this file: algo06_02.c
 *
 * For a demo how to use this algorithm see the file "demo_algo06_02.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double f0 = 0., ah=0., af=0., brh=0., blh=0., (*pdfh)(double);

int setup06_02( double (*pdf)(double), double Afb, double bl, double br)
/*
 * Algorithm 6.2 (Monotone-bounded, p. 128)
 *
 * pdf ... function pointer to the density, which must be monotne on (0,1)
 * Afb ... upper bound for the area below the (quasi-)density 
 * bl  ... left border of the domain
 * br  ... right border of the domain
 *
 */
{    
      pdfh = pdf;
      af = Afb;
      blh = bl;
      brh = br;
/*1*/ f0 = (br-bl)*(*pdf)(bl);
/*2*/ ah = af*(1.+log(f0/af)); 
      printf("Setup:\n  total area below hat: %f\n\n",ah);
      return 1;
} 

double generator06_02()
/* is called after the setup to generate random variates */
{ double u,v,x,h;

/*3*/ while(1){
/*4*/   u = ah*uniform();
        v = uniform();
/*5*/   if(u<= af){
/*6*/     x = u/f0;
          h = f0;
        }
/*7*/   else{
/*8*/     x = (af/f0)*exp(u/af-1.);
          h = af/x;
        }
/*9*/   x = (brh-blh)*x+blh;
/*10*/  if(v*h<=(*pdfh)(x))
/*11*/    return x;
      }
}
