/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.6 (Discrete-Simple-RoU), p. 231
 *
 * this file: algo10_06.c
 *
 * For a demo how to use this algorithm see the file "demo_algo10_06.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

static double vl,vr,al,ar,(*pmfh)(int);
 static int m;


int setup10_06(double (*pmf)(int), int mi, double S, double Fm1)
/*
 * Algorithm 10.6 (Discrete-Simple-RoU), p. 231
 * generates discrete variates
 *
 * pmf... function pointer to unimodal probability (mass) function for k >= 0
 *        it need not sum up to 1.
 * mi ... mode of the pmf
 * S ... sum of the probabilities of the pmf
 * Fm1 ... CDF value at m-1; if unknown -1. must be used
 *
 */
{ 
      double pm;

      m = mi;
      pmfh = pmf;
      pm = (*pmf)(m);
/*1*/ vl = sqrt((*pmf)(m-1));
      vr = sqrt(pm);
/*2*/ if(Fm1>=0.){
/*3*/	al = -Fm1*S;
	ar = (1.-Fm1)*S;
      }
/*4*/ else{
/*5*/	al = -S - pm;
	ar = S;
      }
      return 1;
}

int generator10_06()
/* is called after the setup to generate random variates */
{
      double u, v;
      int ix;
/*6*/ while(1){
/*7*/	u = al+(ar-al)*uniform();
/*8*/	if(u<0.){
/*9*/	  v = vl*uniform();
/*10*/	  u = u/vl;
	}
/*11*/	else{
/*12*/	  v = vr*uniform();
/*13*/	  u = u/vr;
	}
/*14*/	ix = floor(u/v)+m;
/*15*/	if(v*v <= (*pmfh)(ix)) 
/*16*/    return ix;
      }
}



