/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 2.2 (Rejection-from-Uniform-Hat), p. 17 
 *
 * this file: demo_algo02_02.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo02_02 demo_algo02_02.c algo02_02.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution
   you can insert your own distribution here
*/
double my_pdf(double x)
/* triangular on (0,1) */
{ return 2.*x;
}




#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(133248); /*set seed for uniform generator*/

 printf("Demo for Algorithm 2.2\nfirst we display ten numbers generated from my_pdf()\n");
 for(i=0;i<10;i++){
   x = generator02_02(my_pdf,0.,1.,2.);
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d for pdf_tri()\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = generator02_02(pdf_tri,0.,1.,2.);
   obs[(int)(cdf_tri(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

