/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Example 2.19, p. 33, 
 * Cauchy distribution by ratio of uniforms method 
 *
 * this file: example02_19.c
 *
 * to compile this file you can use the command:
 *
 * cc -o example02_19 example02_19.c uniform.c statutil.c -lm
 *
 */


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "statutil.h"
#include "uniform.h"

/**********************************************************************/

double example02_19(void)
/* 
 * generates random variates from the Cauchy distribution 
 * using the ratio of uniforms method 
 */
{ double u,v;

/*1*/ while(1){ 
/*2*/   u = 2*uniform()-1.;
        v = uniform();
/*3*/   if(u*u+v*v<=1.) return u/v;
      }
}

/*****************************************************************/


#define CLAS 500

static long int obs[CLAS*CLAS];

int main()
{ int i;
 double x;

 setseed(13324885); /*set seed for uniform generator*/

 printf("Example 2.19:\n\nfirst we display ten generated numbers\n");

 for(i=0;i<10;i++){
   x = example02_19();
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = example02_19();
   obs[(int)(cdf_cauchy(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}






